/****************************************************************
Project : PPE Robot Solaire
Version : 1
// --------------------------------------------------------------
Fichier source : TWI2V2BATRON.c
Date de  cration : 05/07/2009
Mise  jour le : 14/11/2010
// --------------------------------------------------------------
Author  : Philippe Mariano
Company : Lyce Pierre Emile Martin 18000 BOURGES
// --------------------------------------------------------------
Comments: Mise en oeuvre du protocole I2C sur l'interface TWI
(Adaptation de la note d'application AVR315 d'ATMEL)
Transfert de l'tat des BP sur les LEDs de la carte (PCF8574)
Affichage du nom du BP actionn
// --------------------------------------------------------------
Cross compilateur: CVAVR 2.04.5b
// --------------------------------------------------------------
Chip type           : ATmega8535
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 512
****************************************************************/

#include <mega8535.h>
// =============================================================================================================================
// Bibliothques  rajouter
// -----------------------------------------------------------------------------------------------------------------------------
#include <string.h>				// Fonction de traitement des chanes de caractres
#include <delay.h>
#include <stdio.h>				// Fonction sprintf
#include "DriverTWICV2.h"       // Fonctions du drivers TWI adaptes pour CVAVR
                                // ATTENTION: Vrifier que la taille du buffer est adapte au message (20 par dfaut)
#include "LCDBATRON.h"          // Fonctions de l'afficheur I2C LCD
#include "SISISOL.h"            // Fonction LIRE_BP_I2C()
// =============================================================================================================================
// Constantes
// -----------------------------------------------------------------------------------------------------------------------------
// Messages textuels prdfinis
// -----------------------------------------------------------------------------------------------------------------------------
flash char *Texte_Accueil_Ligne1    = "   PPE Robsol   ";
flash char *Texte_Accueil_Ligne2    = "   TWI2V2BAT.c  ";
flash char *TexteTestBpsurLed	    = "Recopie Bp->Led ";
flash char *Ligne1ChoixEchap		= "        [Echap] ";
flash char *BpPlusSurD0				= "+ -> D0 [Echap] ";
flash char *BpFlecheHSurD1			= "FH-> D1 [Echap] ";
flash char *BpFlecheHSurD2			= "FB-> D2 [Echap] ";
flash char *BpMoinsSurD3			= "- -> D3 [Echap] ";
flash char *BpSetSurD4				= "SET->D4 [Echap] ";
flash char *BpOkSurD5				= "Ok-> D5 [Echap] ";
flash char *BpEntrSurD6				= "ENT->D6 [Echap] ";
flash char *BpEchSurD7				= "ECH->D7 [Echap] ";
// -----------------------------------------------------------------------------------------------------------------------------
// =============================================================================================================================

// =============================================================================================================================
// Prototypes des fonctions en cours de test
// -----------------------------------------------------------------------------------------------------------------------------
// =============================================================================================================================

// *****************************************************************************************************************************
// *****************************************************************************************************************************
// Programme principal
// *****************************************************************************************************************************
// *****************************************************************************************************************************
void main(void)
{
// =============================================================================================================================
// Declare your local variables here
// -----------------------------------------------------------------------------------------------------------------------------
// Divers variables
// -----------------------------------------------------------------------------------------------------------------------------
// Type 		    Nom 				                                    Description
// -----------------------------------------------------------------------------------------------------------------------------
bit BP_I2C=0, BP_I2C_1 = 1;
// -----------------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------
// 1) Messages prdfinis pour les ports I2C PCF8574 associs aux BP et aux LEDs de la carte S_ISI
// -----------------------------------------------------------------------------------------------------------------------------
// Type 		    Nom 				                                    Description
// -----------------------------------------------------------------------------------------------------------------------------
char Frame_I2C_LED_Switching[]={Add_LED_I2C_W,0x00}; 	    // Trame de commande des LED {addLed + W (0x40), data = Led on=0}
char Frame_I2C_LED_All_Off[]={Add_LED_I2C_W,0xff}; 	        // Trame d'extinction des LED {addLed + W (ox40), data = all Led off}
char Frame_I2C_BP_Read[]={Add_BP_I2C_R,0x00}; 		        // Trame de lecture des BP {addBP + R (0x43), data=nothing}
// -----------------------------------------------------------------------------------------------------------------------------
// 2) Messages pour le LCD I2C BATRON de la carte S_ISI
// -----------------------------------------------------------------------------------------------------------------------------
// Type 		Nom 				       Description
// -----------------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------
char Buffer_for_LCD_Line0[17], Buffer_for_LCD_Line1[17];    // Reoivent les deux lignes de texte  envoyer  l'afficheur
                                                            // (16 caractres max. + Nul)
// =============================================================================================================================

// =============================================================================================================================
// Initialisation
// -----------------------------------------------------------------------------------------------------------------------------
TWI_Master_Initialise(); 	                                  // Initialisation de l'interface TWI en mode Standby
// -----------------------------------------------------------------------------------------------------------------------------
// Global enable interrupts
#asm("sei")
// -----------------------------------------------------------------------------------------------------------------------------
// Extinction des LEDs de la carte S_ISI
TWI_Start_Transc_With_Data(Frame_I2C_LED_All_Off,sizeof(Frame_I2C_LED_All_Off));
// -----------------------------------------------------------------------------------------------------------------------------
// Initialisation de l'afficheur BATRON
Init_LCD_I2C_BATRON();
// -----------------------------------------------------------------------------------------------------------------------------
// Affichage d'un message d'accueil
// -----------------------------------------------------------------------------------------------------------------------------
sprintf(Buffer_for_LCD_Line0,Texte_Accueil_Ligne1);
sprintf(Buffer_for_LCD_Line1,Texte_Accueil_Ligne2);
Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
delay_ms(4000);
// =============================================================================================================================

// =============================================================================================================================
// Boucle principale
// =============================================================================================================================
while (1)
      {
// -----------------------------------------  Champ Lecture des donnes ---------------------------------------------------------
// Lecture de l'tat des BP
        TWI_Start_Transc_With_Data(Frame_I2C_BP_Read,1);      	// Envoi de l'octet d'adresse des BP + bit RW=lecture(0x43)
        TWI_Get_Data_From_Transceiver(Frame_I2C_BP_Read,sizeof(Frame_I2C_BP_Read)); // TWI_Get_Data_From_Transceiver(Donnee_BP,2);
        BP_I2C = Lire_BP_I2C();									// Lecture des BP avec gestion anti-rebond
// -----------------------------------------  Champ Traitement ------------------------------------------------------------------
        Frame_I2C_LED_Switching[1] = ~Frame_I2C_BP_Read[1];

         if ((BP_I2C != BP_I2C_1) && (BP_I2C != 0xFF))
				{
				 sprintf(Buffer_for_LCD_Line0,TexteTestBpsurLed);

				 switch (BP_I2C)
				  {
				 	case ENTR_I2C : sprintf(Buffer_for_LCD_Line1,BpEntrSurD6); break;
				 	case FlecheH_I2C : sprintf(Buffer_for_LCD_Line1,BpFlecheHSurD1); break;
				 	case FlecheB_I2C : sprintf(Buffer_for_LCD_Line1,BpFlecheHSurD2); break;
				 	case ECHAP_I2C : sprintf(Buffer_for_LCD_Line1,BpEchSurD7); break;
				 	case OK_I2C : sprintf(Buffer_for_LCD_Line1,BpOkSurD5); break;
				 	case MOINS_I2C: sprintf(Buffer_for_LCD_Line1,BpMoinsSurD3); break;
				 	case PLUS_I2C: sprintf(Buffer_for_LCD_Line1,BpPlusSurD0); break;
				 	case SET_I2C: sprintf(Buffer_for_LCD_Line1,BpSetSurD4); break;
				 	case 0 : sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchap );
				  }

					Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
					BP_I2C_1 = BP_I2C;
			   	 }

        		if (BP_I2C == ECHAP_I2C)
        		  {
        			Frame_I2C_LED_Switching[1] = ~Frame_I2C_BP_Read[1];
                    // Les LED affichent l'tat des BP
        			TWI_Start_Transc_With_Data(Frame_I2C_LED_Switching,sizeof(Frame_I2C_LED_Switching));
        			delay_ms(1000);
        		  }

// -----------------------------------------  Champ Ecriture des rsultats  -----------------------------------------------------
// Ecriture sur les LED
        TWI_Start_Transc_With_Data(Frame_I2C_LED_Switching,sizeof(Frame_I2C_LED_Switching));  // Les LED affichent l'tat des BP
        delay_ms(200);					// Lecture des BP toutes les 200ms
      };
}
// =============================================================================================================================
// *****************************************************************************************************************************

